package main

import "fmt"
import "math"

type Vertex struct {
	X int
	Y int
}


// A method is a function with a special receiver argument
func (this Vertex) Dist() float64 {
	return math.Sqrt(float64(this.X*this.X + this.Y*this.Y))
}

// A method is a function with a special receiver argument
// (need a pointer to change the state... and avoid copying)
// (value methods can be invoked on pointers and values,
// but pointer methods can only be invoked on pointers)
// (When the value is addressable, the language takes care
// of the common case of invoking a pointer method on a value
// by inserting the address operator automatically)
// methods can be invoked on nil objects
func (this *Vertex) Move(dx, dy int) {
	if this != nil {
		this.X += dx
		this.Y += dy
	}
}

func main() {
	v := Vertex{3, 4}
	// method invocation
	fmt.Println(v)
	fmt.Println(v.Dist())
	
	v.Move(6, 8)
	fmt.Println(v)
	fmt.Println(v.Dist())
}



