package main

import "fmt"
import "math"

type Vertex struct {
	X int
	Y int
}

// A method is a function with a special receiver argument
func (this Vertex) Dist() float64 {
	return math.Sqrt(float64(this.X*this.X + this.Y*this.Y))
}

// An interface type is defined as a set of method signatures
type Distable interface {
	Dist() float64
}

type Point int

func (p Point) Dist() float64 {
	return float64(p)
}

func main() {
	var i1, i2 Distable
	//  A variable of interface type can hold any value that implements those methods
	// (don't need to declare that the type implements the interface)
	i1 = Vertex{3, 4}
	i2 = Point(3)
	fmt.Printf("i1.Dist()=%v\n", i1.Dist())
	fmt.Printf("i2.Dist()=%v\n", i2.Dist())

}
