package main

import "time"

func main() {
	println("Hello")
	println("World")
}

func main1() {
	// launch a goroutine
	go println("Hello")
	println("World")
	// Hey, what happens? Where is Hello?
	// (when main ends all its goroutines are terminated)
}

func main2() {
	// launch a goroutine
	go println("Hello")
	println("World")
	time.Sleep(1000)
	// Here is Hello!
}


// let's sync on a channel
func main3() {
	done := make(chan bool)
	// launch a goroutine
	go func() {
		println("Hello")
		done <- true // send value true on channel done
	}()
	println("World")
	// wait on channel done, ignore received value 
	<-done
}


// Hello takes a channel for exchanging booleans
func Hello(done chan bool) {
	println("Hello")
	done <- true // send value true on channel done
}

func main4() {
	// create a channel for sending booleans
	done := make(chan bool)
	go Hello(done) // launch a goroutine
	println("World")
	// wait on channel done, ignore received value 
	<-done // receive a value from channel done
	// this way World may get printed before Hello
}

func main5() {
	done := make(chan bool)
	go Hello(done)
	<-done 
	// this way Hello gets printed before World
	println("World")
}

